/* Copyright (c) 2015-present, salesforce.com, inc. All rights reserved
Licensed under BSD 3-Clause - see LICENSE.txt or git.io/sfdc-license */

import { defaultState, render, setState, state, template } from 'elix/src/base/internal';
import { fragmentFrom } from 'elix/src/core/htmlLiterals';

import SdsButton from '@salesforce-ux/sds-button/button';
import css from './button-toggle.module.css';

const KEYCODE = {
  SPACE: 32,
  ENTER: 13,
};

export default class SdsButtonToggle extends SdsButton {
  get [defaultState]() {
    return Object.assign(super[defaultState], {
      toggle: null,
      pressed: false,
    });
  }

  /**
   * Tracks if the button is pressed or unpressed.
   */
  get toggle() {
    return this[state].toggle;
  }

  set toggle(toggle) {
    this[setState]({
      toggle: toggle,
    });
  }

  /**
   * When a user activates the button and toggles between pressed and unpressed.
   */
  get pressed() {
    return this[state].pressed;
  }

  set pressed(pressed) {
    this[setState]({
      pressed: pressed,
    });
  }

  /**
   * Track keydown for accessibility
   * @param {event} e
   */
  _onKeyDown(e) {
    if (e.altKey) return;
    switch (e.keyCode) {
      case KEYCODE.SPACE:
      case KEYCODE.ENTER:
        e.preventDefault();
        this._togglePressed();
        break;
      default:
        break;
    }
  }

  /**
   * Add attributes based if the user toggles the button
   */
  _togglePressed() {
    this.toggleAttribute('pressed');
    this[setState]({
      ariaPressed: !this[state].ariaPressed,
    });
  }

  [render](changed) {
    super[render](changed);

    const button = this.shadowRoot.querySelector('[part="button"]');

    if (changed.toggle) {
      const { toggle } = this[state];
      if (toggle) {
        delegateAttribute(this, button, 'aria-pressed', this[state].ariaPressed);
        this.addEventListener('keydown', this._onKeyDown);
        this.addEventListener('click', this._togglePressed);
      }
    }
  }

  get [template]() {
    const result = super[template];

    result.content.append(fragmentFrom.html`
    <style>
      ${css}
    </style>
    `);

    return result;
  }
}

function delegateAttribute(from, to, name, value) {
  if (value || typeof value === 'boolean') {
    to.setAttribute(name, value);
    from.removeAttribute(name);
  } else {
    to.removeAttribute(name);
  }
}
